from django import forms
from models import Topic, Message

"""
class TopicForm(forms.Form):
    title       = forms.CharField()
    category    = forms.HiddenInput()
    forum       = forms.HiddenInput()
    title       = forms.CharField()
    # slug        = forms.CharField(null=True, blank=True)
    sticky      = forms.CheckboxInput()
    user        = forms.HiddenInput()
    text        = forms.CharField(widget=forms.Textarea)
"""

class TopicForm(forms.ModelForm):
    class Meta:
        model = Topic
        exclude = ('user', 'category', 'forum', 'last_user', 'message', 'last_message', 'sticky', 'reply_count',
        'view_count', 'locked', 'active', 'created', 'responded_to', 'modified', 'slug')

class MessageForm(forms.ModelForm):
    class Meta:
        model = Message
        exclude = ('category', 'forum', 'topic', 'user', 'created', 'modified', 'active')



from django.db import models
from managers import ForumManager
from django.contrib.auth.models import User, Group
from uuslug import uuslug
import datetime

class Category(models.Model):
    groups              = models.ManyToManyField(Group, null=True, blank=True)
    name                = models.CharField(max_length=255, null=False, blank=False)
    slug                = models.CharField(max_length=255, null=True, blank=True)
    image               = models.URLField(max_length=255, null=True, blank=True)
    description         = models.TextField(null=True, blank=True)
    position            = models.PositiveSmallIntegerField("Position", null=True, blank=True)
    collapsible         = models.BooleanField(default=True)
    active              = models.BooleanField(default=False)
    require_logged_in   = models.BooleanField(default=False)

    class Meta():
        ordering = ['position']
        verbose_name_plural = "Categories"

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.id:
            self.slug = uuslug(self.name, instance=self)
        super(Category, self).save(*args, **kwargs)

class Forum(models.Model):
    groups              = models.ManyToManyField(Group, null=True, blank=True)
    name                = models.CharField(max_length=255, null=False, blank=False)
    slug                = models.CharField(max_length=255, null=True, blank=True)
    description         = models.TextField()
    category            = models.ForeignKey(Category)
    order               = models.IntegerField(null=True, blank=True)
    position            = models.IntegerField(null=True, blank=True)
    created             = models.DateTimeField(null=True, blank=True)
    last_updated        = models.DateTimeField(null=False, blank=False)
    topic_count         = models.IntegerField(null=True, blank=True, default=0)
    message_count       = models.IntegerField(null=True, blank=True, default=0)
    activate            = models.BooleanField(default=False)

    class Meta():
        ordering = ['position']

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.id:
            self.created = datetime.datetime.now()
            self.slug = uuslug(self.name, instance=self)
        super(Forum, self).save(*args, **kwargs)

    def is_moderator(self, *args, **kwargs):
        if kwargs:
            if "user_id" in kwargs:
                mod = self.moderator_set.filter(user__id=kwargs["user_id"]).count()
                return mod == 1
        return False

class Moderator(models.Model):
    user                = models.ForeignKey(User)
    forum               = models.ForeignKey(Forum)

    def __unicode__(self):
        return self.user.username

class ActiveTopicManager(models.Manager):
    def get_query_set(self):
            return super(ActiveTopicManager, self).get_query_set().filter(active=True)

class Topic(models.Model):
    category            = models.ForeignKey(Category, null=True, blank=True)
    forum               = models.ForeignKey(Forum)
    title               = models.CharField(max_length=255, null=False, blank=False)
    slug                = models.CharField(max_length=255, null=True, blank=True)
    sticky              = models.BooleanField(default=False)
    user                = models.ForeignKey(User, related_name='user')
    last_user           = models.ForeignKey(User, related_name='last_user')
    message             = models.ForeignKey('Message', related_name='message', null=True, blank=True)
    last_message        = models.ForeignKey('Message', related_name='last_message', null=True, blank=True)
    reply_count         = models.IntegerField(default=0)
    view_count          = models.IntegerField(default=0)
    locked              = models.BooleanField(default=False)
    active              = models.BooleanField(default=False)
    created             = models.DateTimeField(null=True, blank=True)
    responded_to        = models.DateTimeField(null=True, blank=True)
    modified            = models.DateTimeField(null=True, blank=True)
    # objects             = ActiveTopicManager()
    # all_objects         = models.Manager()
    objects             = models.Manager()

    class Meta():
        get_latest_by   = 'created'
        ordering        = ['created']

    def __unicode__(self):
        return self.title

    def save(self, *args, **kwargs):
        if not self.id:
            self.slug = uuslug(self.title, instance=self)
            self.created = datetime.datetime.now()
        else:
            self.modified = datetime.datetime.now()
        super(Topic, self).save(*args, **kwargs)

    @models.permalink
    def get_absolute_url(self):
        return ('forum_topic_home', [self.category.slug, self.forum.slug, self.slug])

class Message(models.Model):
    category            = models.ForeignKey(Category)
    forum               = models.ForeignKey(Forum)
    topic               = models.ForeignKey(Topic, related_name='topic_messages')
    user                = models.ForeignKey(User)
    text                = models.TextField()                
    created             = models.DateTimeField(null=True, blank=True)
    modified            = models.DateTimeField(null=True, blank=True)
    active              = models.BooleanField(default=True)

    class Meta():
        ordering = ['created']

    def __unicode__(self):  
        return "%s : %s" % (self.user.username, self.topic.title)

    def save(self, *args, **kwargs):
        if not self.id:
            # self.slug = uuslug(self.title, instance=self)
            self.created = datetime.datetime.now()
        else:
            self.modified = datetime.datetime.now()
        super(Message, self).save(*args, **kwargs)



from django import template
from django.core.urlresolvers import reverse

from djero.models import Message, Topic, Category
from django.contrib.auth.models import User
from django.contrib.sessions.models import Session
import datetime

register = template.Library()

class UserCountNode(template.Node):
    def __init__(self):
        pass

    def render(self, context):
        users = User.objects.filter(is_active=True).count()
        return users-1

class TopicCountNode(template.Node):
    def render(self, context):
        topics = Topic.objects.filter(active=False).count()
        return topics

class MessageCountNode(template.Node):
    def render(self, context):
        messages = Message.objects.filter(active=True).count()
        return messages

@register.tag
def user_count(parser, token):
    return UserCountNode()

@register.tag
def topic_count(parser, token):
    return TopicCountNode()

@register.tag
def message_count(parser, token):
    return MessageCountNode()

@register.tag
def is_moderator(parser, token):
    bits = token.contents.split()
    if len(bits) <= 2:
        raise TemplateSyntaxError, "is_moderator needs a forum and user"
    return IsModerator(bits[1], bits[2])

@register.filter
def is_moderator_of(user, forum):
    try: 
        mod = forum.moderator_set.filter(user=user).count()
        print mod
        return mod == 1
    except Exception:
        return False

@register.filter
def is_in_group_for_category(user, category):
    if user.is_superuser == True:
        return True
    try: 
        category = Category.objects.get(pk=category.pk)
        category_groups = category.groups.all()
        if len(category_groups) == 0:
            return True
        passes = False
        for user_group in user.groups.all():
            print user_group
            if user_group in category_groups:
                passes = True
                return True
        return passes
    except Exception, e:
        print str(e)
        return False

@register.filter
def can_see_category(user, category):
    passes = True
    anon = user.is_anonymous()

    if user.is_superuser == True:
        return True
    if category.require_logged_in and anon:
        return False
    return passes

@register.filter
def can_see_forum(user, forum):
    passes = True
    anon = user.is_anonymous()

    if user.is_superuser == True:
        return True
    if forum.require_logged_in and anon:
        return False
    return passes



from django.conf.urls import patterns, include, url

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    url(r'^$',                                          'djero.views.index',        name='forum_home'),
    url(r'^(?P<category>[-\w]+)/$',                     'djero.views.category',     name='forum_category_home'),
    url(r'^subscribe_topic/(?P<topic>[-\w]+)/$',        'djero.views.subscribe_topic',    
                                                                                    name='forum_subscribe_topic'),
    url(r'^subscribe_forum/(?P<topic>[-\w]+)/$',        'djero.views.subscribe_forum',    
                                                                                    name='forum_subscribe_forum'),
    url(r'^topic/new/(?P<forum>[-\w]+)/$',              'djero.views.create_topic', name='forum_create_topic'),
    url(r'^topic/reply/(?P<topic>[-\w]+)/$',            'djero.views.reply',        name='forum_reply'),
    url(r'^(?P<category>[-\w]+)/(?P<forum>[-\w]+)/$',   'djero.views.forum',        name='forum_board_home'),
    url(r'^topic/(?P<topic>[-\w]+)/edit/delete$',       'djero.views.delete_topic', name='forum_delete_topic'),
    url(r'^topic/(?P<topic>[-\w]+)/edit/$',             'djero.views.edit_topic',   name='forum_edit_topic'),
    url(r'^(?P<category>[-\w]+)/(?P<forum>[-\w]+)/(?P<topic>[-\w]+)/$',   
                                                        'djero.views.topic',        name='forum_topic_home'),
)
